#ifndef CGUL_COMMON_H
#define CGUL_COMMON_H

/**
 * Common macros.
 *
 * @file
 * @author Paul Serice
 * @brief common macros
 */

#include <assert.h>
#include <ctype.h>
#include <errno.h>
#include <limits.h>
#include <locale.h>
#include <stdarg.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

/**
 * The cgul version.
 */
#define CGUL_VERSION "1.8.0"

/**
 * The import/export tag for Windows.
 */
#ifndef CGUL_EXPORT
#  ifdef WIN32
#    ifdef libcgul_EXPORTS
#      define CGUL_EXPORT __declspec(dllexport)
#    else
#      define CGUL_EXPORT __declspec(dllimport)
#    endif
#  else
#    define CGUL_EXPORT
#  endif
#endif

#ifdef __cplusplus
#    define CGUL_BEGIN_C extern "C" {
#    define CGUL_END_C }
#else
#    define CGUL_BEGIN_C
#    define CGUL_END_C
#endif

/**
 * Return the minimum of <code>x</code> and <code>y</code>.
 */
#define CGUL_MIN(x, y) (((x) < (y)) ? (x) : (y))

/**
 * Return the maximum of <code>x</code> and <code>y</code>.
 */
#define CGUL_MAX(x, y) (((x) < (y)) ? (y) : (x))

/**
 * This macro will return the number of bytes needed to store "nbits".
 */
#define CGUL_BITS_TO_BYTES(nbits) (((nbits) + (CHAR_BIT - 1)) / CHAR_BIT)

/**
 * Precomputed value of 1/2^32 as a double.  This is used by PRNGs
 * when return values on the range [0, 1).
 */
#define CGUL_INVERSE_2_32 ((double)2.3283064365386963e-10)

#endif  /*  CGUL_COMMON_H  */
