#ifndef CGUL_LIST_NODE_CXX_H
#define CGUL_LIST_NODE_CXX_H

/**
 * C++ wrapper for <code>cgul_list_node</code>.
 *
 * @file
 * @author Paul Serice
 * @brief C++ wrapper for <code>cgul_list_node</code>
 */

#include "cgul_exception_cxx.h"
#include "cgul_list_node.h"

/**
 * This class provides the C++ bindings for C
 * <code>cgul_list_node</code> objects.  The main purpose of this
 * class is to convert the C-style function calls and exception
 * handling in <code>cgul_list_node</code> into C++-style function
 * calls and exception handling.  This class must not be extended.
 *
 * @brief C++ bindings for <code>cgul_list_node</code>
 */
class cgul_list_node_cxx {

public:

    /*
     * IMPLEMENTATION NOTE:
     * ===================
     *
     * There are no real cgul_list_node_cxx objects.  Instead, this
     * class is carefully implemented so that "cgul_list_node"
     * objects (not "cgul_list_node_cxx" objects) can be directly
     * used to invoke the methods of this class.
     *
     * This works because this class has no associated data _and_ no
     * virtual methods which would cause a vtable lookup, and vtables
     * are typically embedded in the object itself.
     *
     * In summary, do _not_ add data or virtual functions to this
     * class.
     */

    friend class cgul_list_cxx;

    /**
     * Return the value stored in the node.
     *
     * @return value stored in <code>n</code>
     */
    void*
    get_value() const
    {
        void* rv = NULL;
        cgul_list_node_t n = (cgul_list_node_t)this;
        cgul_exception_t local = NULL;
        rv = cgul_list_node__get_value(&local, n);
        if (local) {
            throw cgul_exception_cxx(local);
        }
        return rv;
    }

    /**
     * Set the value to be stored in the node.
     *
     * @param[in] value value to be stored in the node
     */
    void
    set_value(const void* value)
    {
        cgul_list_node_t n = (cgul_list_node_t)this;
        cgul_exception_t local = NULL;
        cgul_list_node__set_value(&local, n, value);
        if (local) {
            throw cgul_exception_cxx(local);
        }
    }

    /**
     * Return the next node.  You can use this method to iterate over
     * the list.  When there are no more nodes, <code>NULL</code> is
     * returned.  You can use <code>get_back()</code> to get the last
     * node in order to start iterating.  A <code>set_prev()</code>
     * exists, but it is private.
     *
     * @return previous node
     */
    cgul_list_node_cxx*
    get_prev() const
    {
        cgul_list_node_cxx* rv = NULL;
        cgul_list_node_t n = (cgul_list_node_t)this;
        cgul_exception_t local = NULL;
        rv = (cgul_list_node_cxx*)cgul_list_node__get_prev(&local, n);
        if (local) {
            throw cgul_exception_cxx(local);
        }
        return rv;
    }

    /**
     * Return the previous node.  You can use this method to iterate
     * over the list.  When there are no more nodes, <code>NULL</code>
     * is returned.  You can use <code>get_front()</code> to get the
     * first node in order to start iterating.  A
     * <code>set_next()</code> exists, but it is private.
     *
     * @return next node
     */
    cgul_list_node_cxx*
    get_next() const
    {
        cgul_list_node_cxx* rv = NULL;
        cgul_list_node_t n = (cgul_list_node_t)this;
        cgul_exception_t local = NULL;
        rv = (cgul_list_node_cxx*)cgul_list_node__get_next(&local, n);
        if (local) {
            throw cgul_exception_cxx(local);
        }
        return rv;
    }

    /**
     * Get the underlying <code>cgul_list_node</code> object.
     *
     * @return underlying object
     */
    cgul_list_node_t
    get_obj() const
    {
        /* Because of the special implementation of this class, "this"
         * is in fact the underlying object.  (See the comments at the
         * top of the file.) */
        return (cgul_list_node_t)this;
    }

private:

    /* Hide these. */
    cgul_list_node_cxx();
    cgul_list_node_cxx(const cgul_list_node_cxx&);
    ~cgul_list_node_cxx();
    cgul_list_node_cxx& operator=(const cgul_list_node_cxx&);

};

#endif  /*  CGUL_LIST_NODE_CXX_H  */
