#ifndef CGUL_SPLIT_H
#define CGUL_SPLIT_H

/**
 * Split a string based on a list of character separators.
 *
 * @file
 * @author Paul Serice
 * @brief split a string based on a list of character separators
 */

#include "cgul_common.h"
#include "cgul_exception.h"
#include "cgul_vector.h"

CGUL_BEGIN_C

/**
 * This function splits <code>s</code> into parts.  Each part is
 * separated by one or more consecutive characters from the
 * <code>separators</code> string.  Each part is appended to
 * <code>parts</code> as a new C-style string.  The caller is
 * responsible for calling <code>free()</code> on each part. <p>
 *
 * The <code>allow_empty_parts</code> boolean determines whether this
 * function inserts empty strings to indicate that one separator
 * directly followed another without any non-separator characters in
 * the middle. If this boolean is set to <code>0</code>, any sequence
 * of separators in <code>s</code> will be treated as just a single
 * separator. <p>
 *
 * This function can fail if memory cannot be allocated for the
 * strings that are appended to <code>parts</code>.  If an error
 * occurs, <code>parts</code> will not be altered, and an exception
 * will be thrown. <p>
 *
 * @param[in,out] cex c-style exception
 * @param[in] s string
 * @param[in] separators string of characters that separate parts
 * @param[in] allow_empty_parts whether to allow empty parts
 * @param[out] parts parts of <code>s</code>
 * @see cgul_strtok
 */
extern CGUL_EXPORT void
cgul_split(cgul_exception_t* cex,
           const char* s,
           const char* separators,
           int allow_empty_parts,
           cgul_vector_t parts);

CGUL_END_C

#endif  /*  CGUL_SPLIT_H  */
