#ifndef CGUL_BCD_P_H
#define CGUL_BCD_P_H

#include "cgul_bcd.h"
#include "cgul_string.h"
#include "cgul_big_integer_p.h"

CGUL_BEGIN_C

/** Data encapsulated by the <code>cgul_bcd</code> class. */
struct cgul_bcd {

    /**
     * The <code>cgul_string</code> class already handles memory
     * allocation for a sequence of bytes.  We are simply going to
     * borrow this code and store the binary form of each decimal
     * digit as one <code>char</code> in the string with the
     * least-significant digit stored in <code>digits[0]</code>.  This
     * means we are _not_ going to pack two decimal digits into each
     * byte.  It is safe to store <code>(char)0</code> in a
     * <code>cgul_string</code> because <code>cgul_string</code>
     * tracks its length independently of the trailing null
     * terminator.
     */
    cgul_string_t digits;

    /**
     * <code>is_positive</code> stores the sign of the value.  A value
     * of 1 means the integer is positive.  A value of 0 means the
     * integer is negative.
     */
    int is_positive;

};

/**
 * Assign <code>cbi_element_t</code> in <code>rhs</code> to
 * <code>lhs</code>.  This is a private method designed for use by
 * <code>cgul_big_integer__append_as_decimal()</code> so that it can
 * use this class to quickly convert the binary representation in
 * <code>cgul_big_integer</code> to a "binary-coded decimal"
 * representation which can easily be converted to a string.
 *
 * @param[in,out] cex c-style exception
 * @param[out] lhs left-hand side
 * @param[in] rhs right-hand side
 */
extern CGUL_EXPORT void
cgul_bcd__assign_from_cbi_element(cgul_exception_t* cex,
                                  cgul_bcd_t lhs,
                                  cbi_element_t rhs);

CGUL_END_C

#endif  /*  CGUL_BCD_P_H  */
