#include "cgul_block_reader_memory_p.h"
#include "cgul_libc.h"

/**
 * Virtual Function Table for the <code>cgul_block_reader_memory</code> class.
 */
struct cgul_block_reader_memory__static_vtable {

    /* Declare */

    /**
     * Virtual methods derived from <code>cgul_block_reader</code>.
     */
    struct cgul_block_reader__vtable br_vtable;

} cgul_block_reader_memory__static_vtable_instance = {

    /* Static Initialization */

    /* br_vtable */
    {
        (cgul_block_reader__virtual_delete_t)
        cgul_block_reader_memory__delete,

        (cgul_block_reader__virtual_read_next_block_t)
        cgul_block_reader_memory__read_next_block,

        (cgul_block_reader__virtual_get_presentation_name_t)
        cgul_block_reader_memory__get_presentation_name,

        (cgul_block_reader__virtual_set_presentation_name_t)
        cgul_block_reader_memory__set_presentation_name,
    },

};

/**
 * Common code to avoid memory leaks when setting the presentation name.
 *
 * @param[in,out] cex c-style exception
 * @param[in] brmemory <code>cgul_block_reader_memory</code> instance
 * @param[in] presentation_name presentation name
 */
static void
cgul_block_reader_memory__set_presentation_name__common(
    cgul_exception_t* cex,
    cgul_block_reader_memory_t brmemory,
    const char* presentation_name)
{
    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(brmemory);

    /* Clear the old presentation name. */
    if (brmemory->presentation_name) {
        free(brmemory->presentation_name);
        brmemory->presentation_name = NULL;
    }

    /* Set the new presentation name. */
    brmemory->presentation_name = cgul_libc__strdup(cex, presentation_name);
    if (*cex) {
        goto out;
    }

 out:
    return;
}

/**
 * Initialize or reinitialize the
 * <code>cgul_block_reader_memory</code> object <code>brmemory</code>.
 * If an error occurs, an exception is thrown.
 *
 * @param[in,out] cex c-style exception
 * @param[in] brmemory <code>cgul_block_reader_memory</code> instance
 */
static void
cgul_block_reader_memory__reinitialize(cgul_exception_t* cex,
                                       cgul_block_reader_memory_t brmemory)
{
    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(brmemory);

    /* super -- initialized in constructor */

    /* buffer -- We do not own the input buffer. */
    brmemory->buffer = NULL;
    brmemory->buffer_size = 0;
    brmemory->buffer_offset = 0;

    /* Set the presentation name to its default value. */
    cgul_block_reader_memory__set_presentation_name__common(
        cex, brmemory, "MEMORY");

 out:
    return;
}

cgul_block_reader_t
cgul_block_reader_memory__as_block_reader(cgul_exception_t* cex,
                                          cgul_block_reader_memory_t brmemory)
{
    cgul_block_reader_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(brmemory);

    /* Return pointer to embedded super-class. */
    rv = &brmemory->super;

 out:
    return rv;
}

cgul_block_reader_memory_t
cgul_block_reader_memory__new(cgul_exception_t* cex)
{
    cgul_block_reader_memory_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Allocate. */
    rv = (cgul_block_reader_memory_t)cgul_libc__calloc(cex, 1, sizeof(*rv));
    if (*cex) {
        goto out;
    }

    /* super */
    rv->super.vptr =
        &cgul_block_reader_memory__static_vtable_instance.br_vtable;

    /* Initialize the new object. */
    cgul_block_reader_memory__reinitialize(cex, rv);
    if (*cex) {
        goto out;
    }

 out:

    /* Delete partially instantiated objects. */
    if (*cex && rv) {
        cgul_block_reader_memory__delete(rv);
        rv = NULL;
    }

    return rv;
}

cgul_block_reader_memory_t
cgul_block_reader_memory__new_from_memory(cgul_exception_t* cex,
                                          void* buffer,
                                          size_t buffer_size)
{
    cgul_block_reader_memory_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Create a default object. */
    rv = cgul_block_reader_memory__new(cex);
    if (*cex) {
        goto out;
    }

    /* Open "buffer". */
    cgul_block_reader_memory__open(cex, rv, buffer, buffer_size);
    if (*cex) {
        goto out;
    }

 out:

    /* Delete partially instantiated objects. */
    if (*cex && rv) {
        cgul_block_reader_memory__delete(rv);
        rv = NULL;
    }

    return rv;
}

void
cgul_block_reader_memory__delete(cgul_block_reader_memory_t brmemory)
{
    cgul_exception_t local = NULL;

    if (brmemory) {

        /* Close the current buffer if any. */
        cgul_block_reader_memory__close(&local, brmemory);
        if (local) {
            /* Printing a warning is the best we can do here because
             * we are inside the destructor. */
            const char* msg = cgul_exception__get_message(local);
            fprintf(stderr,
                    "*** Warning: cgul_block_reader_memory__delete: %s\n",
                    msg);
            cgul_exception__delete(local);
            local = NULL;
        }

        /* presentation_name */
        if (brmemory->presentation_name) {
            free(brmemory->presentation_name);
        }

        /* Free the instance itself. */
        free(brmemory);

    }
}

void
cgul_block_reader_memory__open(cgul_exception_t* cex,
                               cgul_block_reader_memory_t brmemory,
                               void* buffer,
                               size_t buffer_size)
{
    int reset_object_on_error = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(brmemory);

    /* If an error occurs, reset the brfile object to its default values. */
    reset_object_on_error = 1;

    /* Sanity check. */
    if (!buffer && (buffer_size > 0)) {
        *cex = cgul_exception__new("cgul_block_reader_memory__open: "
                                   "input buffer is NULL and "
                                   "its size is greater than zero");
        goto out;
    }

    /* Close the current input buffer before opening a new one. */
    cgul_block_reader_memory__close(cex, brmemory);

    /* Set the new input buffer, size, and offset. */
    brmemory->buffer = (unsigned char*)buffer;
    brmemory->buffer_size = buffer_size;
    brmemory->buffer_offset = 0;

 out:

    /* Clean up. */
    if (*cex && reset_object_on_error) {
        cgul_exception_t local = NULL;
        cgul_block_reader_memory__reinitialize(&local, brmemory);
        if (local) {
            /* An exception is already pending the best we can do
             * is print a warning. */
            const char* msg = cgul_exception__get_message(local);
            fprintf(stderr,
                    "Warning: cgul_block_reader_memory__open: "
                    "full object reset failed after failing to open memory: %s",
                    msg);
            cgul_exception__delete(local);
            local = NULL;
        }
    }

    return;
}

void
cgul_block_reader_memory__close(cgul_exception_t* cex,
                                cgul_block_reader_memory_t brmemory)
{
    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(brmemory);

    /* Reinitialize the object. */
    cgul_block_reader_memory__reinitialize(cex, brmemory);
    if (*cex) {
        goto out;
    }

 out:
    return;
}

size_t
cgul_block_reader_memory__read_next_block(cgul_exception_t* cex,
                                          cgul_block_reader_memory_t brmemory,
                                          void* block,
                                          size_t block_size)
{
    size_t rv = 0;
    size_t remainder_size = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(brmemory);

    /* Calculate the size of the bytes remaining in the input buffer. */
    if (brmemory->buffer_size > brmemory->buffer_offset) {
        remainder_size = brmemory->buffer_size - brmemory->buffer_offset;
    }

    /* EOF -- Check for EOF before any other sanity checks because we
     * don't have to access either the input or output buffer in this
     * case. */
    if (remainder_size == 0) {
        goto out;
    }

    /* Sanity check. */
    if (!brmemory->buffer) {
        *cex = cgul_exception__new(
            "cgul_block_reader_memory__read_next_block: "
            "input buffer is NULL");
        goto out;
    }

    /* Sanity check. */
    if (block_size == 0) {
        /* The output buffer has zero length so do not try to write to it. */
        goto out;
    }

    /* Sanity check. */
    if (!block) {
        *cex = cgul_exception__new(
            "cgul_block_reader_memory__read_next_block: "
            "output buffer is NULL");
        goto out;
    }

    /* Determine the number of bytes to read. */
    rv = CGUL_MIN(block_size, remainder_size);

    /* Copy the bytes to the output buffer. */
    memcpy(block, brmemory->buffer + brmemory->buffer_offset, rv);

    /* Move the offset forward. */
    brmemory->buffer_offset += rv;

 out:
    return rv;
}

const char*
cgul_block_reader_memory__get_presentation_name(
    cgul_exception_t* cex,
    cgul_block_reader_memory_t brmemory)
{
    const char* rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(brmemory);

    /* Sanity check. */
    if (!brmemory->buffer) {
        *cex = cgul_exception__new(
            "cgul_block_reader_memory__get_presentation_name: "
            "no input buffer");
        goto out;
    }

    /* Return the presentation name. */
    assert(brmemory->presentation_name);
    rv = brmemory->presentation_name;

 out:
    return rv;
}

void
cgul_block_reader_memory__set_presentation_name(
    cgul_exception_t* cex,
    cgul_block_reader_memory_t brmemory,
    const char* presentation_name)
{
    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Sanity check. */
    if (!brmemory->buffer) {
        *cex = cgul_exception__new(
            "cgul_block_reader_memory__set_presentation_name: "
            "no input buffer");
        goto out;
    }

    /* Set the presentation name. */
    cgul_block_reader_memory__set_presentation_name__common(
        cex, brmemory, presentation_name);
    if (*cex) {
        goto out;
    }

 out:
    return;
}

void*
cgul_block_reader_memory__get_buffer(cgul_exception_t* cex,
                                      cgul_block_reader_memory_t brmemory)
{
    void* rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(brmemory);

    /* Sanity check. */
    if (!brmemory->buffer) {
        *cex = cgul_exception__new("cgul_block_reader_memory__get_buffer: "
                                   "no input buffer");
        goto out;
    }

    /* Return the client's input buffer. */
    rv = (void*)brmemory->buffer;

 out:
    return rv;
}
