#ifndef CGUL_CACHE_P_H
#define CGUL_CACHE_P_H

#include "cgul_common.h"
#include "cgul_cache.h"
#include "cgul_vector.h"

CGUL_BEGIN_C

/**
 * Data encapsulated by the <code>cgul_cache</code> class.
 */
struct cgul_cache {

    /*
     * NOTE: If you modify this structure, make sure you update
     * <code>cgul_cache__the_freer</code> to match.
     */

    /** The maximum number of objects to cache. */
    unsigned long int size;

    /** The number of objects currently reserved by the cache. */
    unsigned long int reserve;

    /**
     * In order for the cache to be useful, the cache itself must not
     * thrash malloc() thus making a vector a good choice for holding
     * the objects that are cached.
     */
    cgul_vector_t v;

    /**
     * Constructor for objects to be cached.  This is specified by the
     * client.
     */
    cgul_cache__constructor_t constructor;

    /**
     * Destructor for objects to be cached.  This is specified by the
     * client.
     */
    cgul_cache__destructor_t destructor;

    /**
     * Restructor for objects to be cached.  This is specified by the
     * client.  A "restructor" knows how to take a fully constructed
     * object and return it to its initial state.
     */
    cgul_cache__restructor_t restructor;

};

CGUL_END_C

#endif  /*  CGUL_CACHE_P_H  */
