#include "cgul_defensive.h"

char
cgul_defensive__add_char(cgul_exception_t* cex,
                         char x1,
                         char x2)
{
    char rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Detect whether "char" is signed or unsigned (which is
     * implementation dependent) and forward the call to the correct
     * function. */
#if CHAR_MAX == UCHAR_MAX
    rv = cgul_defensive__add_unsigned_char(cex, x1, x2);
#elif CHAR_MAX == SCHAR_MAX
    rv = cgul_defensive__add_signed_char(cex, x1, x2);
#else
#    error "Unable to determine whether char is signed or unsigned"
#endif

 out:
    return rv;
}

signed char
cgul_defensive__add_signed_char(cgul_exception_t* cex,
                                signed char x1,
                                signed char x2)
{
    signed char rv = 0;
    signed char tmp = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Test for overflow or for a negative addend. */
    tmp = x1 + x2;
    if ((tmp < x1) || (tmp < x2)) {
        *cex = cgul_exception__new("cgul_defensive__add_signed_char: "
                                   "overflow or negative addend detected: "
                                   "%d + %d = %d",
                                   (int)x1, (int)x2, (int)tmp);
        goto out;
    }

    /* Success. */
    rv = tmp;

 out:
    return rv;
}

unsigned char
cgul_defensive__add_unsigned_char(cgul_exception_t* cex,
                                  unsigned char x1,
                                  unsigned char x2)
{
    unsigned char rv = 0;
    unsigned char tmp = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Test for overflow. */
    tmp = x1 + x2;
    if ((tmp < x1) || (tmp < x2)) {
        *cex = cgul_exception__new("cgul_defensive__add_unsigned_char: "
                                   "overflow detected: "
                                   "%d + %d = %d",
                                   (int)x1, (int)x2, (int)tmp);
        goto out;
    }

    /* Success. */
    rv = tmp;

 out:
    return rv;
}

short
cgul_defensive__add_short(cgul_exception_t* cex,
                          short x1,
                          short x2)
{
    short rv = 0;
    short tmp = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Test for overflow or for a negative addend. */
    tmp = x1 + x2;
    if ((tmp < x1) || (tmp < x2)) {
        *cex = cgul_exception__new("cgul_defensive__add_short: "
                                   "overflow or negative addend detected: "
                                   "%hd + %hd = %hd",
                                   x1, x2, tmp);
        goto out;
    }

    /* Success. */
    rv = tmp;

 out:
    return rv;
}

unsigned short
cgul_defensive__add_unsigned_short(cgul_exception_t* cex,
                                   unsigned short x1,
                                   unsigned short x2)
{
    unsigned short rv = 0;
    unsigned short tmp = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Test for overflow. */
    tmp = x1 + x2;
    if ((tmp < x1) || (tmp < x2)) {
        *cex = cgul_exception__new("cgul_defensive__add_unsigned_short: "
                                   "overflow detected: "
                                   "%hu + %hu = %hu",
                                   x1, x2, tmp);
        goto out;
    }

    /* Success. */
    rv = tmp;

 out:
    return rv;
}

int
cgul_defensive__add_int(cgul_exception_t* cex,
                        int x1,
                        int x2)
{
    int rv = 0;
    int tmp = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Test for overflow or for a negative addend. */
    tmp = x1 + x2;
    if ((tmp < x1) || (tmp < x2)) {
        *cex = cgul_exception__new("cgul_defensive__add_int: "
                                   "overflow or negative addend detected: "
                                   "%d + %d = %d",
                                   x1, x2, tmp);
        goto out;
    }

    /* Success. */
    rv = tmp;

 out:
    return rv;
}

unsigned int
cgul_defensive__add_unsigned(cgul_exception_t* cex,
                             unsigned int x1,
                             unsigned int x2)
{
    unsigned int rv = 0;
    unsigned int tmp = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Test for overflow. */
    tmp = x1 + x2;
    if ((tmp < x1) || (tmp < x2)) {
        *cex = cgul_exception__new("cgul_defensive__add_unsigned: "
                                   "overflow detected: "
                                   "%u + %u = %u",
                                   x1, x2, tmp);
        goto out;
    }

    /* Success. */
    rv = tmp;

 out:
    return rv;
}

long
cgul_defensive__add_long(cgul_exception_t* cex,
                         long x1,
                         long x2)
{
    long rv = 0;
    long tmp = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Test for overflow or for a negative addend. */
    tmp = x1 + x2;
    if ((tmp < x1) || (tmp < x2)) {
        *cex = cgul_exception__new("cgul_defensive__add_long: "
                                   "overflow or negative addend detected: "
                                   "%ld + %ld = %ld",
                                   x1, x2, tmp);
        goto out;
    }

    /* Success. */
    rv = tmp;

 out:
    return rv;
}

unsigned long
cgul_defensive__add_unsigned_long(cgul_exception_t* cex,
                                  unsigned long x1,
                                  unsigned long x2)
{
    unsigned long rv = 0;
    unsigned long tmp = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Test for overflow. */
    tmp = x1 + x2;
    if ((tmp < x1) || (tmp < x2)) {
        *cex = cgul_exception__new("cgul_defensive__add_unsigned_long: "
                                   "overflow detected: "
                                   "%lu + %lu = %lu",
                                   x1, x2, tmp);
        goto out;
    }

    /* Success. */
    rv = tmp;

 out:
    return rv;
}

size_t
cgul_defensive__add_size_type(cgul_exception_t* cex,
                              size_t x1,
                              size_t x2)
{
    size_t rv = 0;
    size_t tmp = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Test for overflow. */
    tmp = x1 + x2;
    if ((tmp < x1) || (tmp < x2)) {
        *cex = cgul_exception__new("cgul_defensive__add_size_type: "
                                   "overflow detected");
        goto out;
    }

    /* Success. */
    rv = tmp;

 out:
    return rv;
}
