#include "cgul_exception_p.h"
#include "cgul_libc.h"

static struct cgul_exception cgul_exception__out_of_memory_exception = {
    (char*)"out of memory"
};

cgul_exception_t
cgul_exception__get_out_of_memory_exception(void)
{
    return &cgul_exception__out_of_memory_exception;
}

cgul_exception_t
cgul_exception__new(const char* format, ...)
{
    cgul_exception_t rv = NULL;
    va_list args;

    assert(format);

    va_start(args, format);
    rv = cgul_exception__new_from_va_list(format, args);
    va_end(args);

    return rv;
}

cgul_exception_t
cgul_exception__new_from_va_list(const char* format,
                                 va_list args)
{
    cgul_exception_t rv = NULL;
    cgul_exception_t local = NULL;

    assert(format);

    /* Create a new exception.  This does not lead to infinite
     * recursion when we are out of memory because cgul_libc__calloc()
     * will return the out of memory exception rather than trying to
     * create a new exception by calling cgul_exception__new()
     * again. */
    rv = (cgul_exception_t)cgul_libc__calloc(&local, 1, sizeof(*rv));
    if (local) {
        goto out;
    }

    rv->message = cgul_libc__vasprintf(&local, format, args);
    if (local) {
        goto out;
    }

 out:

    /* If an exception occurred above, it means we are out of memory.
     * So, delete the partially initialzed object and use the
     * out-of-memory exception instead. */
    if (local) {
        if (rv) {
            cgul_exception__delete(rv);
        }
        /* Override the exception to report the problem we had
         * constructing the exception. */
        rv = local;
    }

    return rv;
}

void
cgul_exception__delete(cgul_exception_t cex)
{
    if (cex) {
        /* Check to make sure you do not call free() on the special
         * object when throwing an exception in an out-of-memory
         * condition. */
        if (cex != &cgul_exception__out_of_memory_exception) {
            if (cex->message) {
                free(cex->message);
            }
            free(cex);
        }
    }
}

const char*
cgul_exception__get_message(cgul_exception_t cex)
{
    assert(cex);
    return cex->message;
}
