#include <iostream>
#include "cgul_cxx.h"

using namespace std;

int
main(int argc,
     char* argv[])
{
    int rv = 0;

    try {

        /* Verify the command line. */
        if (argc != 2) {
            cerr << "Usage: main_bdf_cxx <font.bdf>" << endl;
            return 1;
        }

        /* Allocate a new font object. */
        cgul_bdf_cxx bdf;

        /* Allocate the index_file_name and generate the name of the index
         * file from the name of the BDF file. */
        cgul_string_cxx index_file_name(argv[1]);
        index_file_name.append_char('i');

        try {
            /* Load the font from file.  Try to use the index file if
             * it exists. */
            bdf.load_explicit(argv[1], index_file_name.get_value());
        } catch (const cgul_exception_cxx& e) {
            /* Print the error message. */
            cerr << "***" << endl;
            cerr << "*** Warning: " << e.get_message() << endl;
            cerr << "***" << endl;

            /* Try again but without the index file. */
            cerr << "***" << endl;
            cerr << "*** Trying without index file." << endl;
            cerr << "***" << endl;
            bdf.load(argv[1]);
        }

#if 0
        /* Shift the font right so that no character has a negative
         * x-offset.  This is useful if you want to make sure the first
         * character on the line does not clip on the left side. */
        bdf.shift_right();

        /* Some broken font drawing mechanisms ignore the x-offset
         * component of the bounding box, and draw all glyphs starting at
         * the origin.  We can compensate for this shifting the entire
         * font right (which is done above when calling
         * cgul_bdf__shift_right()) and then adding padding on the left
         * side of each glyph so that the x-offset is always zero. */
        bdf.pad_left();
#endif

        cgul_bdf_glyph_cxx* glyph = bdf.get_glyph((cgul_wchar_t)'A',
                                                   1 /* substitute */);
        if (glyph) {
            glyph->fdraw(stdout);
        }

        /* Write the font to a temporary file. */
        cout << "Creating sample BDF: tmp.bdf" << endl;
        bdf.write("tmp.bdf");

        /* Write the font index to a temporary file. */
        cout << "Creating sample BDF index: tmp.bdfi" << endl;
        bdf.write_index("tmp.bdfi");

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (cgul_exception_cxx e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    return rv;
}
