/*
 * This program raises a base to an exponent.  Most of the time is
 * actually spent converting the large result to decimal.
 */

#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    cgul_big_integer_t a = NULL;
    cgul_big_integer_t b = NULL;
    cgul_big_integer_t p = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    a = cgul_big_integer__new_from_string(&local, "87");
    b = cgul_big_integer__new_from_string(&local, "11039");
    p = cgul_big_integer__new(&local);
    if (local) {
        goto out;
    }

    /* Calculate p = a^b. */
    cgul_big_integer__power(&local, p, a, b);
    if(local) {
        goto out;
    }

    /* Print the result. */
    printf("a = ");
    cgul_big_integer__print_as_decimal(&local, a, stdout);
    puts("");

    printf("b = ");
    cgul_big_integer__print_as_decimal(&local, b, stdout);
    puts("");

    printf("p = ");
    cgul_big_integer__print_as_decimal(&local, p, stdout);
    puts("");

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (p) {
        cgul_big_integer__delete(p);
    }
    if (b) {
        cgul_big_integer__delete(b);
    }
    if (a) {
        cgul_big_integer__delete(a);
    }

    return rv;
}
