#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    size_t rcount = 0;
    size_t block_size = 16384;
    unsigned char* buf = NULL;    
    cgul_block_reader_t br = NULL;
    cgul_block_reader_file_t brfile = NULL;
    cgul_exception_t local = NULL;

    /* Validate the command line. */
    if (argc != 2) {
        fprintf(stderr, "Usage: main_block_reader_file <file>\n");
        goto out;
    }

    /* Allocate. */
    buf = cgul_libc__calloc(&local, 1, block_size);
    brfile = cgul_block_reader_file__new_from_fname(&local, argv[1]);
    if (local) {
        goto out;
    }

    /* Upcast brfile to br. */
    br = cgul_block_reader_file__as_block_reader(&local, brfile);

    for (;;) {

        /* Polymorphically read the next block of data using the
         * cgul_block_reader interface. */
        rcount = cgul_block_reader__read_next_block(&local,
                                                    br,
                                                    buf,
                                                    block_size);
        if (local) {
            goto out;
        }

        /* EOF */
        if (rcount == 0) {
            break;
        }

        /* Write the block to stdout. */
        cgul_libc__fwrite_block(&local, buf, rcount, stdout);
        if (local) {
            goto out;
        }

    }

 out:

    /* Handle exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (br) {
        cgul_block_reader__delete(br);
    }
    if (buf) {
        free(buf);
    }

    return rv;
}
