#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    size_t rcount = 0;
    size_t block_size = 1;
    unsigned char* block = NULL;    
    cgul_block_reader_t br = NULL;
    cgul_block_reader_memory_t brmemory = NULL;
    cgul_exception_t local = NULL;

    const char* test_string = "Hello, World!\n";

    /* Allocate. */
    block = cgul_libc__calloc(&local, 1, block_size);
    brmemory = cgul_block_reader_memory__new_from_memory(
                 &local, (void*)test_string, strlen(test_string));
    if (local) {
        goto out;
    }

    /* Upcast brmemory to br. */
    br = cgul_block_reader_memory__as_block_reader(&local, brmemory);

    for (;;) {

        /* Polymorphically read the next block of data using the
         * cgul_block_reader interface. */
        rcount = cgul_block_reader__read_next_block(&local,
                                                    br,
                                                    block,
                                                    block_size);
        if (local) {
            goto out;
        }

        /* EOF */
        if (rcount == 0) {
            break;
        }

        /* Write the block to stdout. */
        cgul_libc__fwrite_block(&local, block, rcount, stdout);
        if (local) {
            goto out;
        }

    }

 out:

    /* Handle exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (br) {
        cgul_block_reader__delete(br);
    }
    if (block) {
        free(block);
    }

    return rv;
}
