/*
 * This program tests whether cgul_csv works with character-oriented
 * or line-oriented devices like the terminal.  To test with this
 * program, run it from the terminal and copy one line at a time of a
 * CSV file to its standard input.  It should immediately print
 * results once the current CSV record is complete.
 */

#include <iostream>
#include <stdexcept>
#include "cgul_cxx.h"

using namespace std;

/**
 * Print the rows and columns in the CSV file <code>csv</code>.
 *
 * @param[in] csv CSV file
 * @param[in] strm output stream
 */
static void
print(cgul_csv_cxx& csv,
      ostream& strm)
{
    unsigned long int i = 0;
    unsigned long int n = 0;
    const char* field = NULL;

    /* Iterate over each record. */
    while (csv.load_next()) {

        /* Test getting the field by the header for its column. */
        field = csv.lookup_by_header("header1");
        if (field) {
            strm << "field[\"header1\"] = " << field << endl;
        }

        /* Iterate over each field. */
        n = csv.get_number_of_fields();
        for (i = 0 ; i < n ; ++i) {
            field = csv.lookup(i);
            strm << "field[" << i << "] = " << field << endl;
        }

        strm << "=====" << endl;
    }
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;

    try {

        /* Open stdin and print the results. */
        cgul_csv_cxx csv(stdin, 0 /* is_block_buffered */);
        print(csv, cout);

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    return rv;
}
