#include <iostream>
#include <stdexcept>
#include "cgul_cxx.h"

using namespace std;

static int
print_key(const void* key,
          void* data)
{
    printf("key = %s\n", (char*)key);
    return 1;
}

static int
print_value(void* value,
            void* data)
{
    printf("value = %d\n", *(int*)value);
    return 1;
}

static int
print_pair(const void* key,
           void* value,
           void* data)
{
    printf("key = %s  value = %d\n", (char*)key, *(int*)value);
    return 1;
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    int i = 0;
    char* key = NULL;
    int* value = NULL;
    cgul_hash_cxx h;
    cgul_hash_node_cxx* n;

    try {

        /* Populate the hash. */
        for (i = 0 ; i < 5 ; ++i) {
            /* Allocate a new key and value to add to the hash. */
            key = (char*)calloc(1, sizeof(char) * 2);
            value = (int*)calloc(1, sizeof(*value));
            if (!key || !value) {
                throw runtime_error("out of memory");
            }
            /* Assign values to the newly allocated memory. */
            *key = 'a' + i;
            *value = i;
            /* Insert the key/value pair into the hash. */
            h.insert(key, &n);
            n->set_value(value);
        }

        /* Print the keys in forward and then reverse order. */
        h.foldl_keys(print_key, NULL);
        h.foldr_keys(print_key, NULL);

        puts("=====");

        /* Print the values in forward and then reverse order. */
        h.foldl_values(print_value, NULL);
        h.foldr_values(print_value, NULL);

        puts("=====");

        /* Print the pairs in forward and then reverse order. */
        h.foldl_pairs(print_pair, NULL);
        h.foldr_pairs(print_pair, NULL);

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    /* Clean up. */
    h.free_keys();
    h.free_values();

    return rv;
}
