/**
 * This program generates 250K lines of random c-style strings.  These
 * strings are not limited to the ascii character set and are not
 * unicode strings.  They are just random characters except all the
 * '\0' characters have been replaced by '\n' characters.
 */

#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    int i = 0;
    unsigned long int c = 0;
    cgul_random_bbs_t prng = NULL;
    cgul_big_integer_t max = NULL;
    cgul_big_integer_t next = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    prng = cgul_random_bbs__new(&local);
    max = cgul_big_integer__new_from_string(&local, "256");
    next = cgul_big_integer__new(&local);
    if (local) {
        goto out;
    }

    /* Generate 250K lines of c-style strings.  The only thing
     * different about this from having all random characters is that
     * each zero that is generated is replaced with '\n' so that the
     * file can be directly read into a C program. */
    while (i < 256000) {
        cgul_random_bbs__next_in_range(&local, prng, next, max);
        if (!cgul_big_integer__as_unsigned_long(&local, next, &c)) {
            local = cgul_exception__new("out of range");
            goto out;
        }
        /* Check for end-of-string and convert to end-of-line. */
        if (c == 0) {
            c = '\n';
        }
        /* Count lines. */
        if (c == '\n') {
            ++i;
        }
        /* Print this character. */
        putc((int)c, stdout);
    }

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (max) {
        cgul_big_integer__delete(max);
    }
    if (next) {
        cgul_big_integer__delete(next);
    }
    if (prng) {
        cgul_random_bbs__delete(prng);
    }

    return rv;
}
