/**
 * This program generates 10M random unsigned 32-bit values and writes
 * them as text to standard output.
 */

#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    size_t i = 0;
    cgul_uint32_t next = 0;
    cgul_random_xorwow32_t prng = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    prng = cgul_random_xorwow32__new(&local);
    if (local) {
        goto out;
    }

    /* Generate 10M random unsigned 32-bit values and write them to
     * standard output as text. */
    for (i = 0 ; i < 10000000 ; ++i) {
        next = cgul_random_xorwow32__next(&local, prng);
        printf("%#010x   bit1: %u\n", next, next & 0x1);
    }

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (prng) {
        cgul_random_xorwow32__delete(prng);
    }

    return rv;
}
