#include "cgul.h"

int main(int argc, char* argv[])
{
    int rv = 0;
    cgul_exception_t local = NULL;
    cgul_rbtree_t tree = NULL;
    cgul_rbtree_node_t node = NULL;
    cgul_cache_t keys_cache = NULL;
    cgul_cache_t values_cache = NULL;
    cgul_string_t key = NULL;
    cgul_string_t value = NULL;

    keys_cache =
        cgul_cache__new(
            &local,
            (cgul_cache__constructor_t)&cgul_string__new,
            (cgul_cache__destructor_t)&cgul_string__delete,
            (cgul_cache__restructor_t)&cgul_string__clear);

    values_cache =
        cgul_cache__new(
            &local,
            (cgul_cache__constructor_t)&cgul_string__new,
            (cgul_cache__destructor_t)&cgul_string__delete,
            (cgul_cache__restructor_t)&cgul_string__clear);

    tree = cgul_rbtree__new(&local,
                             (cgul_rbtree__compare_t)&cgul_string__compare);

    key = cgul_cache__get(&local, keys_cache);
    cgul_string__set_value(&local, key, "Hello, World!");
    if (cgul_rbtree__insert(&local, tree, key, &node)) {
        value = cgul_cache__get(&local, values_cache);
        cgul_string__set_value(&local, value, "Goodbye, Cruel World!");
        cgul_rbtree_node__set_value(&local, node, value);
    }

    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    if (tree) {
        cgul_rbtree__clear(&local, tree, keys_cache, values_cache);
        cgul_rbtree__delete(tree);
    }

    if (values_cache) {
        cgul_cache__delete(values_cache);
    }

    if (keys_cache) {
        cgul_cache__delete(keys_cache);
    }

    return rv;
}
