#include "cgul_cxx.h"
#include <iostream>
#include <stdexcept>

using namespace std;

/**
 * Print the keys in the tree <code>t</code>.
 *
 * @param[in] cex c-style exception
 * @param[in] t <code>cgul_rbtree</code> instance
 */
static void
main__print_keys(cgul_rbtree_cxx& t)
{
    const char* key = NULL;
    cgul_rbtree_node_cxx* curr = NULL;

    /* Iterate over each key. */
    curr = t.get_front();
    for ( ; curr ; curr = curr->get_next()) {
        key = (const char*)curr->get_key();
        printf("%s\n", key);
    }

}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    cgul_rbtree_cxx t1((cgul_rbtree_cxx::compare_t)strcmp);
    cgul_rbtree_cxx t2((cgul_rbtree_cxx::compare_t)strcmp);

    try {

        /* Insert keys into t1. */
        t1.insert("aaa", NULL);
        t1.insert("bbb", NULL);
        t1.insert("ccc", NULL);

        /* Insert keys into t2. */
        t2.insert("xxx", NULL);
        t2.insert("yyy", NULL);
        t2.insert("zzz", NULL);

        /* Print keys originally in t1. */
        printf("===== t1 (before)\n");
        main__print_keys(t1);

        /* Print keys originally in t2. */
        printf("===== t2 (before)\n");
        main__print_keys(t2);

        /* Swap */
        t1.swap(t2);

        /* Print keys originally in t1. */
        printf("===== t1 (after)\n");
        main__print_keys(t1);

        /* Print keys originally in t2. */
        printf("===== t2 (after)\n");
        main__print_keys(t2);

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    return rv;
}
