#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    cgul_wchar_t wc = CGUL_WCHAR__NUL;
    cgul_block_reader_file_t br = NULL;
    cgul_stream_reader_t sr = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    br = cgul_block_reader_file__new_from_fname(&local, "tower_of_babel.txt");
    sr = cgul_stream_reader__new(&local);
    if (local) {
        goto out;
    }

    /* Set up sr to decode ASCII. */
    cgul_stream_reader__set_decoder(
        &local, sr, cgul_stream_reader__decode_ascii);
    if (local) {
        goto out;
    }

    /* Reduce the block size to exercise stitching blocks together. */
    if (!cgul_stream_reader__set_block_size(&local, sr, 17)) {
        local = cgul_exception__new("Unable to change block size.");
        goto out;
    }

    /* Add the block reader. */
    cgul_stream_reader__add_block_reader(
        &local, sr, cgul_block_reader_file__as_block_reader(&local, br));
    if (local) {
        goto out;
    }

    for (;;) {

        /* Read the next character from the ASCII stream. */
        if (!cgul_stream_reader__read(&local, sr, &wc)) {
            if (local) {
                goto out;
            }
            /* EOF */
            break;
        }

        /* Print the ASCII character. */
        printf("%c", (char)wc);
        fflush(stdout);
    }

 out:

    /* Handle exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (sr) {
        cgul_stream_reader__delete(sr);
    }
    if (br) {
        cgul_block_reader_file__delete(br);
    }

    return rv;
}
