#include <iostream>
#include <stdexcept>
#include "cgul_cxx.h"

using namespace std;

/**
 * This function uses <code>stanza</code> to lookup the key
 * <code>varname</code>.  If the lookup is successful, it appends the
 * result to <code>dst</code>.
 *
 * @param[in,out] cex c-style exception
 * @param[in] varname variable name
 * @param[in,out] dst destination string
 * @param[in] stanza <code>cgul_stanza</code> object
 * @return whether the lookup was successful
 */
static int
do_lookup(const char* varname,
          cgul_string_cxx* dst,
          cgul_stanza_cxx* stanza)
{
    int rv = 0;
    const char* value = NULL;

    /* Perform the lookup. */
    value = stanza->lookup(varname);
    if (value) {
        /* Perform the substitution by appending value to dst. */
        dst->append(value);
        /* Success. */
        rv = 1;
    }
    return rv;
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    const char* src = NULL;
    cgul_string_cxx dst;
    cgul_crlf_file_cxx f;
    cgul_crlf_file_cxx crlf;
    cgul_stanza_cxx stanza;

    try {

        /* Read the mapping of variable names to values. */
        f.open_fname("main_varsub_mapping.txt");
        stanza.open_crlf_file(f);
        stanza.all_one_stanza(1);
        stanza.load_next();

        /* Open the text file. */
        crlf.open_fname("main_varsub_text.txt");

        /* Iterate over each line in the text file. */
        while ((src = crlf.get_line()) != NULL) {

            /* Clear dst. */
            dst.set_value("");

            /* Perform variable substitution. */
            cgul_varsub_cxx(src,
                            &dst,
                            (cgul_varsub_cxx__lookup_t)do_lookup,
                            &stanza);

            /* Print results. */
            cout << dst.get_value() << endl;
        }

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    return rv;
}
