#include "cgul.h"

struct point {
    int x;
    int y;
};

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    size_t i;
    size_t v_count = 1000000;
    struct point* pt = NULL;
    cgul_vector_t v = NULL;
    cgul_random_lfg32_t prng = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    v = cgul_vector__new(&local);
    prng = cgul_random_lfg32__new(&local);
    if (local) {
        goto out;
    }

    /* Initialize. */
    for (i = 0 ; i < v_count ; ++i) {
        /* Allocate and initialize a new point. */
        pt = cgul_libc__calloc(&local, 1, sizeof(*pt));
        if (local) {
            goto out;
        }
        pt->x = i;
        pt->y = i + 1;

        /* Add the point to the vector. */
        cgul_vector__push_back(&local, v, pt);
        if (local) {
            goto out;
        }

        /* The vector now owns the point. */
        pt = NULL;
    }

    /* Shuffle. */
    cgul_vector__shuffle(&local,
                         v,
                         (cgul_shuffle__next_in_range_t)
                             cgul_random_lfg32__next_in_range,
                         prng);
    if (local) {
        goto out;
    }

#if 0
    {
        size_t i_max = 0;
        struct point** arr = NULL;

        /* Print the shuffled array. */
        arr = (struct point**)cgul_vector__get_array(&local, v);
        i_max = cgul_vector__get_size(&local, v);
        for (i = 0 ; i < i_max ; ++i) {
            printf("(%6d, %6d)\n", arr[i]->x, arr[i]->y);
        }
    }
#endif

 out:

    /* Handle exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (pt) {
        free(pt);
    }
    if (prng) {
        cgul_random_lfg32__delete(prng);
    }
    if (v) {
        cgul_vector__free_values(v);
        cgul_vector__delete(v);
    }

    return rv;
}
