#include <iostream>
#include <stdexcept>
#include "cgul_cxx.h"

using namespace std;

/**
 * Append the <code>unsigned long</code> values on the half-open range
 * <code>[start, stop)</code> to the vector <code>v</code>.  The
 * client is responsible for calling <code>free()</code> on all the
 * pointers added to <code>v</code>.  If an error occurs, an exception
 * is thrown.
 *
 * @param[in] v vector
 * @param[in] start inclusive starting value
 * @param[in] stop exclusive stopping value
 */
static void
append_values(cgul_vector_cxx& v,
              unsigned long int start,
              unsigned long int stop)
{
    unsigned long int i = 0;
    unsigned long int* n = NULL;

    /* Iterate over the values */
    for (i = start ; i < stop ; ++i) {

        /* Allocate space for the value. */ 
        n = (long unsigned int*)cgul_libc_cxx::calloc(1, sizeof(*n));

        /* Set the value. */
        *n = i;

        /* Append the value to the vector. */
        v.push_back(n);
    }
}

#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    unsigned long int i = 0;
    unsigned long int** ns = NULL;
    cgul_vector_cxx v;

    try {

        /* Add [0 - 5) to the v. */
        append_values(v, 0, 5);

        /* Append a trailing sentinel. */
        v.push_back(NULL);

        /* Take the underlying array. */
        ns = (unsigned long int**)v.take_array();

        /* Print ns. */
        for (i = 0 ; ns[i] ; ++i) {
            cout << "ns[" << i << "] = " << *ns[i] << endl;
        }

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    }

    /* Clean up. */
    v.free_values();
    for (i = 0 ; ns[i] ; ++i) {
        free(ns[i]);
    }
    free(ns);

    return rv;
}
