#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    size_t i = 0;
    cgul_string_t utf8 = NULL;
    cgul_wstring_t utf32 = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    utf8 = cgul_string__new(&local);
    utf32 = cgul_wstring__new(&local);
    if (local) {
        goto out;
    }

#if 0
    {
        cgul_wchar_t hello[] = {'h', 'e', 'l', 'l', 'o', CGUL_WCHAR__NUL};
        cgul_wstring__set_value(&local, utf32, hello);
        if (local) {
            goto out;
        }
    }
#endif

    /* Set the minimum capacity to 2 (one character plus NUL-terminator). */
    cgul_wstring__set_minimum_capacity(&local, utf32, 2);
    if (local) {
        goto out;
    }

    /* Create the string one character at a time.  Note that the final
     * string needs a capacity of (16 + 1) including the
     * NUL-terminator. */
    for (i = 0 ; i < 16 ; ++i) {

        /* Append the next character. */
        cgul_wstring__append_char(&local, utf32, 'z');
        if (local) {
            goto out;
        }

        /* Convert from UTF-32 to UTF-8. */
        cgul_string__set_length(&local, utf8, 0);
        cgul_unicode__wcstombs(&local,
                               cgul_wstring__get_value(&local, utf32),
                               cgul_wstring__get_length(&local, utf32),
                               /* skip_bom */ 1,
                               utf8);
        if (local) {
            goto out;
        }

        /* Print the new value of the string. */
        printf("-->%s<--\n", cgul_string__get_value(&local, utf8));
    }

    /* Reduce the size of the string one character at a time. */
    for (i = cgul_wstring__get_length(&local, utf32) ; i > 0 ; --i) {

        /* Reduce the size of the string by one by removing the
         * trailing character. */
        cgul_wstring__set_length(&local, utf32, i - 1);

        /* Convert from UTF-32 to UTF-8. */
        cgul_string__set_length(&local, utf8, 0);
        cgul_unicode__wcstombs(&local,
                               cgul_wstring__get_value(&local, utf32),
                               cgul_wstring__get_length(&local, utf32),
                               /* skip_bom */ 1,
                               utf8);
        if (local) {
            goto out;
        }

        /* Print the new value of the string. */
        printf("-->%s<--\n", cgul_string__get_value(&local, utf8));
    }

 out:

    /* Handle exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (utf32) {
        cgul_wstring__delete(utf32);
    }
    if (utf8) {
        cgul_string__delete(utf8);
    }

    return rv;
}
